<?php

namespace Drupal\media_migration\Plugin\migrate\source\d7;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Database\Query\SelectInterface;
use Drupal\media\MediaTypeInterface;
use Drupal\migrate_drupal\Plugin\migrate\source\DrupalSqlBase;

/**
 * Drupal 7 media source field instances source based on source database.
 *
 * @MigrateSource(
 *   id = "d7_media_source_field_instance",
 *   source_module = "file_entity"
 * )
 */
class MediaSourceFieldInstance extends DrupalSqlBase {

  use MediaFieldQueryTrait;

  /**
   * The map of the alt and title properties and their corresponding field name.
   *
   * These property fields only exist when the file_entity module is installed
   * on the source site.
   *
   * @var string[]
   */
  protected $fieldNamePropertyMap = [
    'field_file_image_alt_text' => 'alt_field',
    'field_file_image_title_text' => 'title_field',
  ];

  /**
   * {@inheritdoc}
   */
  public function query() {
    // Unused. All of the required data of the single row is collected in
    // ::initializeIterator().
    return NULL;
  }

  /**
   * Adds tags and metadata to a query.
   *
   * @param \Drupal\Core\Database\Query\SelectInterface $query
   *   The plain query to prepare.
   *
   * @return \Drupal\Core\Database\Query\SelectInterface
   *   The query with additional tags and metadata.
   *
   * @see Drupal\migrate\Plugin\migrate\source\SqlBase::prepareQuery()
   */
  protected function prepareGivenQuery(SelectInterface $query) {
    $query->addTag('migrate');
    $query->addTag('migrate_' . $this->migration->id());
    $query->addMetaData('migration', $this->migration);

    return $query;
  }

  /**
   * {@inheritdoc}
   */
  protected function initializeIterator() {
    $empty_iterator = new \ArrayIterator();

    // @todo Remove checking of $this->entityManager and
    //   $this->entityTypeManager and leave only $this->entityManager when
    //   Drupal core 8.7.x security support ends.
    $manager = $this->entityTypeManager ?? $this->entityManager ?? NULL;
    $media_type = NULL;

    try {
      $media_type = $manager->getStorage('media_type')
        ->load('image');
    }
    // Catch plugin not found and invalid plugin definition exceptions.
    catch (PluginNotFoundException $e) {
      return $empty_iterator;
    }
    catch (InvalidPluginDefinitionException $e) {
      return $empty_iterator;
    }

    // If this media type does not exist yet or $media_type is not a media type,
    // we have nothing to migrate.
    if (empty($media_type) || !($media_type instanceof MediaTypeInterface)) {
      return $empty_iterator;
    }

    // Pre-populate the row with the required field config values.
    $row = [
      'field_name' => $media_type->getSource()->getConfiguration()['source_field'],
      'entity_type' => 'media',
      'bundle' => 'image',
    ];

    // Alt and title properties of an image field always exist, regardless
    // of whether they are accessible and editable on the entity edit form or
    // not.
    // But we also want to make sure that if either the alt or title property
    // was editable on the source site, then it will be editable also on the
    // destination site.
    // For first, we try to determine which properties should be enabled (and
    // thus accessible and editable on UI) based on the source media image
    // entities' alt and title fields' content. We assume that if there is at
    // least a single value for an image's title property in the source
    // database, then we have to change visibility of the title property to
    // TRUE, so that it can be edited on the destination site as well.
    $alt_title_config = $this->getSettingsFromPropertyFieldContent();
    // All image field content is also migrated into a media entity. If any of
    // the alt or title properties shouldn't be necessarily shown (and be
    // editable) on the entity edit form based on the source media entity's alt
    // or title field value, we still have to check the configuration of the
    // source site's image fields.
    // If any of the preexisting image field was configured to show the alt or
    // the title property, then we will make their input field visible.
    if (!empty(array_diff(array_values($this->fieldNamePropertyMap), array_keys($alt_title_config)))) {
      $alt_title_config += $this->getSettingsFromImageFields();
    }

    // Get the 'required' settings from the image fields we found.
    $this->mergePropertyRequiredSettingsFromPropertyFields($alt_title_config);

    // Add the discovered and the default configuration.
    $row += $alt_title_config + [
      'alt_field' => TRUE,
      'alt_field_required' => TRUE,
      'title_field' => FALSE,
      'title_field_required' => FALSE,
    ];

    return new \ArrayIterator([$row]);
  }

  /**
   * {@inheritdoc}
   */
  public function count($refresh = FALSE) {
    return $this->initializeIterator()->count();
  }

  /**
   * {@inheritdoc}
   */
  public function fields() {
    return [
      'field_name' => $this->t('The field name.'),
      'entity_type' => $this->t('The entity type.'),
      'bundle' => $this->t('The entity bundle.'),
      'alt_field' => $this->t('Alt field is enabled.'),
      'alt_field_required' => $this->t('Alt field is required.'),
      'title_field' => $this->t('Title field is enabled.'),
      'title_field_required' => $this->t('Title field is required.'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getIds() {
    return [
      'entity_type' => [
        'type' => 'string',
        'alias' => 'msfci',
      ],
      'bundle' => [
        'type' => 'string',
        'alias' => 'msfci',
      ],
      'field_name' => [
        'type' => 'string',
        'alias' => 'msfci',
      ],
    ];
  }

  /**
   * Discovers the enabled properties based on the source fields.
   *
   * The alt and title properties of an image field always exist, regardless
   * of whether they are actually accessible and editable on the entity edit
   * form or not. This method checks the content of the corresponding alt and
   * title fields. If we find at least a single, non-empty row, we say that the
   * actual property should be shown on the destination media entity's edit
   * form.
   *
   * @return true[]
   *   An array of those field instance settings that should be revealed, keyed
   *   by the settings key ('alt_field', 'title_field').
   *   For example, if we have rows for alt, but don't have any data for title,
   *   the array returned will be this:
   *   @code
   *   [
   *     'alt_field' => TRUE
   *   ]
   *   @endcode
   */
  protected function getSettingsFromPropertyFieldContent() {
    $data = [];

    foreach ($this->fieldNamePropertyMap as $field_name => $property) {
      $property_values_query = $this->getDatabase()->select("field_data_$field_name", $field_name)
        ->fields($field_name)
        ->condition("$field_name.{$field_name}_value", '', '<>')
        ->isNotNull("$field_name.{$field_name}_value");
      $property_values_present = (int) $property_values_query->countQuery()->execute()->fetchField() > 0;

      if ($property_values_present) {
        $data[$property] = TRUE;
      }
    }

    return $data;
  }

  /**
   * Discovers enabled properties based on the image field configurations.
   *
   * The alt and title properties of an image field always exist, regardless
   * of that they are actually accessible and editable on the entity edit form.
   * This method checks the config of every image field's instance configuration
   * When alt (or title) was enabled for at least one image field, we say that
   * the property should be shown on the destination media entity's edit
   * form.
   *
   * @return array
   *   An array of those settings that should be revealed, keyed by the settings
   *   key ('alt_field', 'title_field').
   */
  protected function getSettingsFromImageFields() {
    $data = [];
    $image_field_names = $this->getImageFieldNames();

    foreach ($image_field_names as $image_field_name) {
      $field_instance_config_results = $this->getDatabase()->select('field_config_instance', 'fci')
        ->fields('fci', ['data'])
        ->condition('fci.field_name', $image_field_name)
        ->condition('fci.entity_type', 'file', '<>')
        ->execute()
        ->fetchAll();

      foreach ($field_instance_config_results as $field_instance_config_result) {
        $field_config_data = unserialize($field_instance_config_result->data);

        foreach ($this->fieldNamePropertyMap as $property) {
          if (isset($field_config_data['settings'][$property]) && !empty($field_config_data['settings'][$property])) {
            $data[$property] = TRUE;
          }
        }

        if (empty(array_diff(array_values($this->fieldNamePropertyMap), array_keys($data)))) {
          break 2;
        }
      }
    }

    return $data;
  }

  /**
   * Merges 'required' settings based in image field discovery.
   *
   * If any image field have alt (or title) set up as optional, we don't let
   * them being required.
   *
   * @param array $data
   *   The discovered data about alt and title revealment.
   */
  protected function mergePropertyRequiredSettingsFromPropertyFields(array &$data) {
    foreach ($this->fieldNamePropertyMap as $field_name => $property) {
      $property_field_config_result = $this->getDatabase()->select('field_config_instance', 'fci')
        ->fields('fci', ['data'])
        ->condition('fci.field_name', $field_name)
        ->condition('fci.bundle', 'image')
        ->condition('fci.entity_type', 'file')
        ->execute()
        ->fetchAll();

      assert(count($property_field_config_result) === 1);

      if ($property_field_config_data = unserialize($property_field_config_result[0]->data)) {
        if (isset($property_field_config_data['required'])) {
          $not_set_or_not_required = !isset($data["{$property}_required"]) || empty($data["{$property}_required"]);
          $data["{$property}_required"] = $not_set_or_not_required && !empty($data[$property]) && !empty($property_field_config_data['required']);
        }
      }
    }
  }

}
